@@ -27,6 +27,7 @@
 #include <string.h>
 #include <glade/glade-xml.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-help.h>
 #include <gconf/gconf-client.h>
 
 #include "netstatus-icon.h"
@@ -320,9 +321,48 @@ netstatus_dialog_set_iface_name (NetstatusDialogData *data,
 }
 
 static void
-netstatus_dialog_response (GtkWidget *dialog)
+netstatus_dialog_response (GtkWidget *dialog,
+			   int        response)
 {
-  gtk_widget_hide (dialog);
+  switch (response)
+    {
+    case GTK_RESPONSE_HELP:
+      {
+	GError    *error = NULL;
+	GdkScreen *screen;
+
+	screen = gtk_window_get_screen (GTK_WINDOW (dialog));
+
+	gnome_help_display_on_screen ("gnome-netstatus", NULL, screen, &error);
+
+	if (error)
+	  {
+	    GtkWidget *message_dialog;
+                                                                              
+	    message_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
+						     GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("There was an error displaying help:\n%s"),
+						     error->message);
+
+	    gtk_window_set_resizable (GTK_WINDOW (message_dialog), FALSE);
+	    gtk_window_set_screen (GTK_WINDOW (message_dialog), screen);
+	    
+	    g_signal_connect (message_dialog, "response",
+			      G_CALLBACK (gtk_widget_destroy),
+			      NULL);
+	    
+	    gtk_widget_show (message_dialog);
+
+	    g_error_free (error);
+	  }
+      }
+      break;
+    default:
+      gtk_widget_hide (dialog);
+      break;
+    }
 }
 
 static void
