@@ -38,6 +38,7 @@ struct _NetstatusIconPrivate
   NetstatusIface *iface;
   NetstatusState  state;
 
+  GtkIconTheme   *icon_theme;
   GdkPixbuf      *icons [NETSTATUS_STATE_LAST];
   GdkPixbuf      *scaled_icons [NETSTATUS_STATE_LAST];
 
@@ -64,6 +65,11 @@ enum {
   PROP_TOOLTIPS_ENABLED
 };
 
+static void netstatus_icon_init_pixbufs (NetstatusIcon *icon);
+static void netstatus_icon_scale_icons  (NetstatusIcon *icon,
+					 int            width,
+					 int            height);
+
 static GObjectClass *parent_class;
 static guint icon_signals [LAST_SIGNAL] = { 0 };
 
@@ -89,24 +95,100 @@ netstatus_icon_get_default_pixbuf (NetstatusIcon *icon)
 }
 
 static void
-netstatus_icon_init_pixbuf (NetstatusIcon  *icon,
-			    GdkPixbuf     **pixbuf,
-			    const char     *filename)
+netstatus_icon_theme_changed (NetstatusIcon *icon,
+			      GtkIconTheme  *icon_theme)
 {
-  GError *error;
+  int i;
 
-  error = NULL;
-  *pixbuf = gdk_pixbuf_new_from_file (filename, &error);
-  if (error)
+  for (i = 0; i < NETSTATUS_STATE_LAST; i++)
     {
-      netstatus_adopt_error (error, NETSTATUS_ERROR_ICONS);
-      netstatus_iface_set_error (icon->priv->iface, error);
-      g_error_free (error);
-      
-      *pixbuf = netstatus_icon_get_default_pixbuf (icon);
+      g_object_unref (icon->priv->scaled_icons [i]);
+      icon->priv->scaled_icons [i] = NULL;
+
+      g_object_unref (icon->priv->icons [i]);
+      icon->priv->icons [i] = NULL;
     }
 
-  g_assert (*pixbuf != NULL);
+  netstatus_icon_init_pixbufs (icon);
+
+  if (icon->priv->size)
+    {
+      netstatus_icon_scale_icons (icon,
+				  icon->priv->size,
+				  icon->priv->size);
+    }
+}
+
+static GtkIconTheme *
+netstatus_icon_get_icon_theme (NetstatusIcon *icon)
+{
+  if (!icon->priv->icon_theme)
+    {
+      GdkScreen *screen;
+
+      screen = gtk_widget_get_screen (GTK_WIDGET (icon));
+      icon->priv->icon_theme = gtk_icon_theme_get_for_screen (screen);
+
+      g_signal_connect_object (icon->priv->icon_theme, "changed",
+			       G_CALLBACK (netstatus_icon_theme_changed),
+			       icon,
+			       G_CONNECT_SWAPPED);
+    }
+
+  return icon->priv->icon_theme;
+}
+
+static char *
+netstatus_icon_lookup_icon_theme (NetstatusIcon *icon,
+				  const char    *icon_name)
+{
+  GtkIconTheme *icon_theme;
+  GtkIconInfo  *icon_info;
+  char         *filename = NULL;
+
+  icon_theme = netstatus_icon_get_icon_theme (icon);
+
+  if ((icon_info = gtk_icon_theme_lookup_icon (icon_theme, icon_name, 1000, 0)))
+    {
+      filename = g_strdup (gtk_icon_info_get_filename (icon_info));
+
+      gtk_icon_info_free (icon_info);
+    }
+
+  return filename;
+}
+
+static void
+netstatus_icon_init_pixbuf (NetstatusIcon  *icon,
+			    GdkPixbuf     **pixbuf,
+			    const char     *icon_name)
+{
+  char *filename;
+
+  g_assert (*pixbuf == NULL);
+
+  if ((filename = netstatus_icon_lookup_icon_theme (icon, icon_name)))
+    {
+      GError *error;
+
+      error = NULL;
+      *pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+      if (error)
+	{
+	  netstatus_adopt_error (error, NETSTATUS_ERROR_ICONS);
+	  netstatus_iface_set_error (icon->priv->iface, error);
+	  g_error_free (error);
+      
+	}
+
+      g_free (filename);
+    }
+
+  if (!*pixbuf)
+    {
+      *pixbuf = netstatus_icon_get_default_pixbuf (icon);
+      g_assert (*pixbuf != NULL);
+    }
 }
 
 static void
@@ -114,32 +196,32 @@ netstatus_icon_init_pixbufs (NetstatusIcon *icon)
 {
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_DISCONNECTED],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-disconn.png");
+			      "gnome-netstatus-disconn");
   icon->priv->scaled_icons [NETSTATUS_STATE_DISCONNECTED] = NULL;
 
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_IDLE],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-idle.png");
+			      "gnome-netstatus-idle");
   icon->priv->scaled_icons [NETSTATUS_STATE_IDLE] = NULL;
 
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_TX],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-tx.png");
+			      "gnome-netstatus-tx");
   icon->priv->scaled_icons [NETSTATUS_STATE_TX] = NULL;
 
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_RX],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-rx.png");
+			      "gnome-netstatus-rx");
   icon->priv->scaled_icons [NETSTATUS_STATE_RX] = NULL;
 
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_TX_RX],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-txrx.png");
+			      "gnome-netstatus-txrx");
   icon->priv->scaled_icons [NETSTATUS_STATE_TX_RX] = NULL;
 
   netstatus_icon_init_pixbuf (icon,
 			      &icon->priv->icons [NETSTATUS_STATE_ERROR],
-			      NETSTATUS_ICONDIR "/gnome-netstatus-error.png");
+			      "gnome-netstatus-error");
   icon->priv->scaled_icons [NETSTATUS_STATE_ERROR] = NULL;
 }
 
